#include <glib-object.h>
#include <gtk/gtk.h>

#include "app.h"
#include "page.h"

enum {
	PROP_WIDGET_EMPTY,
	PROP_PAGE_LIST
};

G_DEFINE_TYPE(WPSApp, wps_app, G_TYPE_OBJECT);

void button_exit_cb(GtkAction *action, WPSApp *app);
void view_forward_cb(GtkAction *action, WPSApp *app);
void view_back_cb(GtkAction *action, WPSApp *app);
void full_screen_cb(GtkToggleAction *action, WPSApp *app);
void about_cb(GtkAction *action, WPSApp *app);

static GtkActionEntry menu_entry [] = {
        { "File", NULL, "File",
        	NULL,
                NULL,
                NULL},
        { "Edit", NULL, "Edit",
        	NULL,
                NULL,
                NULL},
        { "View", NULL, "View",
        	NULL,
                NULL,
                NULL},
	{ "HelpMenu", NULL, "Help",
        	NULL,
                NULL,
                NULL},
	{ "FileQuit", GTK_STOCK_QUIT, "Quit",
		"<control>Q",
		NULL,
		G_CALLBACK(button_exit_cb)},
	{ "EditPreferences", GTK_STOCK_PREFERENCES, "Preferences",
		"<control>P",
		NULL,
		G_CALLBACK(button_exit_cb)},
	{ "ViewBack", GTK_STOCK_GO_BACK, "Back",
		NULL,
		NULL,
		G_CALLBACK(view_back_cb)},
	{ "ViewForward", GTK_STOCK_GO_FORWARD, "Forward",
		NULL,
		NULL,
		G_CALLBACK(view_forward_cb)},
	{ "ViewUp", GTK_STOCK_GO_UP, "Up",
		NULL,
		NULL,
		G_CALLBACK(button_exit_cb)},
	{ "HelpAbout", GTK_STOCK_ABOUT, "About",
		NULL,
		NULL,
		G_CALLBACK(about_cb)}
	};

static GtkToggleActionEntry menu_toggle_entry [] = {
        { "ViewFullScreen", GTK_STOCK_FULLSCREEN, "Full Screen",
		"<control>F",
		NULL,
		G_CALLBACK(full_screen_cb),
		FALSE}
	};

void about_cb(GtkAction *action, WPSApp *app)
{
	static const gchar copyright[] =
		"Copyright \xc2\xa9 2009 TEST, Inc.";
	
	gtk_show_about_dialog(GTK_WINDOW(app->win),
			       "name", g_get_application_name(),
			       "comments", "HMI screens.",
			       "wrap-license", TRUE,
			       "copyright", copyright,
			       NULL);
}

void button_exit_cb(GtkAction *action, WPSApp *app)
{
	g_warning("Sleep\n");
	gtk_main_quit();
}

void full_screen_cb(GtkToggleAction *action, WPSApp *app)
{
	if (gtk_toggle_action_get_active(action)) {
		gtk_widget_hide(app->menubar);
		gtk_widget_hide(app->toolbar);
		gtk_window_fullscreen(GTK_WINDOW(app->win));
	} else {
		gtk_widget_show(app->menubar);
		gtk_widget_show(app->toolbar);
		gtk_window_unfullscreen(GTK_WINDOW(app->win));
	}
}

void view_forward_cb(GtkAction *action, WPSApp *app)
{
	gint page_n, max_n;
	
	max_n = gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook));
	page_n = gtk_notebook_get_current_page(GTK_NOTEBOOK(app->notebook));
	if (page_n < --max_n) 
		gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), ++page_n);

}

void view_back_cb(GtkAction *action, WPSApp *app)
{
	gint page_n;
	
	page_n = gtk_notebook_get_current_page(GTK_NOTEBOOK(app->notebook));
	if (page_n > 0) 
		gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), --page_n);
}

void change_tab_cb(	GtkNotebook *notebook,
			GtkNotebookPage *page,
			guint page_num,
			gpointer user_data)
{	
	GList *last_page = NULL;
	WPSApp *app = (WPSApp *)user_data;
	gint count = 0;
	GtkAction *back;
	GtkAction *forward;
	gint max_n;
	
	for (last_page = g_list_last(app->page_list); last_page != NULL; last_page = g_list_previous(last_page)) {
		if (page_num == count)			
			app->current_page = last_page->data;
		count++;
	}
	
	max_n = gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook));
	
	back = gtk_ui_manager_get_action(app->ui_manager, "/MenuBar/ViewMenu/ViewBack");
	forward = gtk_ui_manager_get_action(app->ui_manager, "/MenuBar/ViewMenu/ViewForward");
	
	if (page_num == 0) {		
		gtk_action_set_sensitive(back, FALSE);
		if (!gtk_action_is_sensitive(forward))
			gtk_action_set_sensitive(forward, TRUE);
	} else if (page_num == --max_n) {
		gtk_action_set_sensitive(forward, FALSE);
		if (!gtk_action_is_sensitive(back))
			gtk_action_set_sensitive(back, TRUE);
	} else {
		if (!gtk_action_is_sensitive(back))
			gtk_action_set_sensitive(back, TRUE);
			
		if (!gtk_action_is_sensitive(forward))
			gtk_action_set_sensitive(forward, TRUE);
		
	}
	
	g_warning("%d\n", page_num);
}

static GObject *wps_app_constructor(	GType type,
					guint n_construct_properties,
					GObjectConstructParam *construct_properties)
{
	GObject *obj;
	WPSApp *app;
	GtkActionGroup *action_group;
	GList *last_page;
	WPSPage *page;
	WPSPageClass *page_class;
	
	GtkWidget *scrolled;
	GtkWidget *tab_label;
	
	
	obj = G_OBJECT_CLASS(wps_app_parent_class)->constructor(type, n_construct_properties, construct_properties);
	app = WPS_APP(obj);
	
	app->ui_manager = gtk_ui_manager_new();
	action_group = gtk_action_group_new("MainAction");
	
	gtk_action_group_add_actions(action_group, menu_entry, G_N_ELEMENTS(menu_entry), app);
	gtk_action_group_add_toggle_actions(action_group, menu_toggle_entry, G_N_ELEMENTS(menu_toggle_entry), app);
	gtk_ui_manager_insert_action_group(app->ui_manager, action_group, 0);
	
	app->win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(app->win), 1024, 768);
	
	gtk_ui_manager_add_ui_from_file(app->ui_manager, "ui.xml", NULL);
	gtk_window_add_accel_group(GTK_WINDOW (app->win), gtk_ui_manager_get_accel_group (app->ui_manager));
	
	app->vbox = gtk_vbox_new(FALSE, 0);
	app->menubar = gtk_ui_manager_get_widget(app->ui_manager, "/MenuBar");
	app->toolbar = gtk_ui_manager_get_widget(app->ui_manager, "/ToolBar");
	app->notebook = gtk_notebook_new();
	app->status_bar = gtk_statusbar_new();

	last_page = g_list_last(app->page_list);
	app->current_page = last_page->data;
	
	/* page content */	
	for (last_page = g_list_last(app->page_list); last_page != NULL; last_page = g_list_previous(last_page)) {
		page = last_page->data;
		page_class = WPS_PAGE_GET_CLASS(page);
		scrolled = gtk_scrolled_window_new(NULL, NULL);
		page_class->open_page((GObject *)scrolled, page);
		tab_label = gtk_label_new(page->page_info->name->str);
		gtk_notebook_append_page(GTK_NOTEBOOK(app->notebook), scrolled, tab_label);
	}
	
	g_signal_connect(app->notebook, "switch-page", G_CALLBACK(change_tab_cb), app);
	
	gtk_box_pack_start (GTK_BOX (app->vbox), app->menubar, FALSE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (app->vbox), app->toolbar, FALSE, TRUE, 1);
	gtk_box_pack_start (GTK_BOX (app->vbox), app->notebook, TRUE, TRUE, 2);
	gtk_box_pack_start (GTK_BOX (app->vbox), app->status_bar, FALSE, TRUE, 3);
	gtk_container_add(GTK_CONTAINER(app->win), app->vbox);
	
	gtk_widget_show_all(app->win);
	return obj;
}

static void wps_app_init(WPSApp *app)
{
	app->menubar = NULL;
	app->notebook = NULL;
	app->status_bar = NULL;
	app->toolbar = NULL;
	app->vbox = NULL;
	app->ui_manager = NULL;
	app->page_list = NULL;
}


static void wps_app_get_property(	GObject *object,
					guint property_id,
					GValue *value,
					GParamSpec *pspec)
{
	WPSApp *app = (WPSApp *)object;
	
	switch (property_id) {
		case PROP_PAGE_LIST:
			g_value_set_pointer(value, app->page_list);
			break;
		default:
			break;
	}
}

static void wps_app_set_property(	GObject *object,
					guint property_id,
					const GValue *value,
					GParamSpec *pspec)
{
	WPSApp *app = (WPSApp *)object;
	
	switch (property_id) {
		case PROP_PAGE_LIST:
			app->page_list = g_value_get_pointer(value);
			break;		
		default:
			break;
	}
}

static void wps_app_class_init(WPSAppClass *klass)
{
	GObjectClass *object_class;
	
	object_class = G_OBJECT_CLASS(klass);	
	object_class->constructor = wps_app_constructor;
	object_class->get_property = wps_app_get_property;
	object_class->set_property = wps_app_set_property; 
	
	g_object_class_install_property(object_class,
					PROP_PAGE_LIST,
					g_param_spec_pointer("page_list", 
					"PageList",
					"WPS Page list",
					G_PARAM_READWRITE |
					G_PARAM_CONSTRUCT));
	
}

